from typing import Any, Dict, List

from evrouting.utils import PriorityQueue
from evrouting.T import SoC, Time
from evrouting.charge.T import Label, SoCFunction
from evrouting.charge.factories import SoCFunctionFactory


class LabelPriorityQueue(PriorityQueue):
    def __init__(self, f_soc: SoCFunctionFactory, l_set: List[Label]):
        super().__init__()
        self.f_soc_factory: SoCFunctionFactory = f_soc
        self.l_set: List[Label] = l_set

    def insert(self, label: Label):
        """Breaking ties with lowest soc at t_min."""
        super().insert(item=label, **keys(self.f_soc_factory(label)))

        if self.peak_min() == label:
            self.dominance_check()

    def delete_min(self) -> Any:
        min_label = super().delete_min()
        self.dominance_check()
        return min_label

    def dominance_check(self):
        try:
            label: Label = self.peak_min()
        except KeyError:
            return

        soc = self.f_soc_factory(label)

        # Remove item if it gets dominanted by any label in l_set
        if any(self.f_soc_factory(label) > soc for label in self.l_set):
            self.remove_item(label)


def keys(f_soc: SoCFunction) -> Dict:
    t_min: Time = f_soc.minimum
    soc_min: SoC = f_soc(t_min)
    return {'priority': t_min, 'count': soc_min}
