from evrouting.charge import shortest_path

from ..config import (
    edge_case,
    edge_case_start_node_no_cs,
    edge_case_a_slow,
    init_config
)


class TestRoutes:

    def test_shortest_path_charge_at_s_and_a(self):
        """Charging at s."""
        path = shortest_path(**init_config(edge_case))

        assert path == 3.5

    def test_shortest_path_charge_at_s_only(self):
        """Charging at s."""
        path = shortest_path(**init_config(edge_case_a_slow))

        assert path == 3

    def test_shortest_path_no_charge_s_path_t(self):
        """No charging at s but enough initial SoC to go to t directly."""
        conf = init_config(edge_case_start_node_no_cs)
        conf['initial_soc'] = 4
        path = shortest_path(**conf)

        assert path == 1

    def test_shortest_path_no_charge_s_path_a(self):
        """No charging at s but just enough SoC to go to t via a."""
        conf = init_config(edge_case_start_node_no_cs)
        conf['initial_soc'] = 2
        path = shortest_path(**conf)

        assert path == 2


class TestWithFinalSoC:

    def test_shortest_path_charge_at_s_and_a(self):
        """Charging at s."""
        conf = init_config(edge_case)
        conf['final_soc'] = 3
        path = shortest_path(**conf)

        assert path == 5

    def test_path_impossilbe(self):
        """Not possible to end with full battery."""
        conf = init_config(edge_case)
        conf['final_soc'] = 4
        path = shortest_path(**conf)

        assert path is None

    def test_shortest_path_charge_at_s_only(self):
        """Charging at s and a to reach final_soc."""
        conf = init_config(edge_case_a_slow)
        conf['final_soc'] = 3
        path = shortest_path(**conf)

        assert path == 5

    def test_shortest_path_no_charge_s_path_t(self):
        """No charging at s but initial soc."""
        conf = init_config(edge_case_start_node_no_cs)
        conf['initial_soc'] = 4
        conf['final_soc'] = 3
        path = shortest_path(**conf)

        assert path == 2.5
